<?php
/**
 * @file
 * Exports that help Drupal developers when working with fields.
 */

/**
 * Export individual field and field instance configurations.
 */
function fieldhelp_field_export_list() {
  $instances = field_info_instances();
  $field_types = field_info_field_types();
  $bundles = field_info_bundles();

  $modules = system_rebuild_module_data();

  $header = array(t('Field'), t('Field type'), t('Instances'));
  $rows = array();
  foreach ($instances as $entity_type => $type_bundles) {
    foreach ($type_bundles as $bundle => $bundle_instances) {
      foreach ($bundle_instances as $field_name => $instance) {
        $field = field_info_field($field_name);

        // Initialize the row if we encounter the field for the first time.
        if (!isset($rows[$field_name])) {
          $rows[$field_name]['data'][0] = l($field_name, 'admin/reports/fields/export/field/' . $field_name);
          $module_name = $field_types[$field['type']]['module'];
          $rows[$field_name]['data'][1] = $field_types[$field['type']]['label'] . ' ' . t('(module: !module)', array('!module' => $modules[$module_name]->info['name']));
        }

        // Add the current instance.
        $rows[$field_name]['data'][2][] = l($bundles[$entity_type][$bundle]['label'], 'admin/reports/fields/export/instance/' . $field_name . '/' . $entity_type . '/' . $bundle);
      }
    }
  }
  foreach ($rows as $field_name => $cell) {
    $rows[$field_name]['data'][2] = implode(', ', $cell['data'][2]);
  }
  if (empty($rows)) {
    $output = t('No fields have been defined yet.');
  }
  else {
    // Sort rows by field name.
    ksort($rows);
    $output = theme('table', array('header' => $header, 'rows' => $rows));
  }
  return $output;
}

/**
 * Page callback for exporting a given field instance.
 */
function fieldhelp_field_export_instance_page($field_name, $entity_type, $bundle) {
  $build = array();
  $instance = fieldhelp_field_export_instance($entity_type, $field_name, $bundle);
  include_once DRUPAL_ROOT . '/includes/utility.inc';
  $build['data'] = array(
    '#prefix' => '<pre>',
    '#markup' => drupal_var_export($instance),
    '#suffix' => '</pre>',
  );
  $build['instructions'] = array(
    '#prefix' => '<p>',
    '#markup' => t('The below command can be used in an install profile or update hook to create a field instance of an existing or <a href="@url">created field</a>.  To place this field on a different instance, simply change the value of the "bundle" key.', array('@url' => url('admin/reports/fields/export/field/' . $field_name))),
    '#suffix' => '</p>',
  );
  $build['command'] = array(
    '#prefix' => '<textarea cols="100" rows="12" class="form-textarea">',
    '#markup' => '$instance = ' . drupal_var_export($instance) . ';' . "\n" . 'field_create_instance($instance);',
    '#suffix' => '</textarea>',
  );
  return $build;
}

/**
 * Page callback for exporting a given field definition.
 */
function fieldhelp_field_export_field_page($field_name) {
  $build = array();
  $field = fieldhelp_field_export_field($field_name);
  include_once DRUPAL_ROOT . '/includes/utility.inc';
  $build['data'] = array(
    '#prefix' => '<pre>',
    '#markup' => drupal_var_export($field),
    '#suffix' => '</pre>',
  );
  $build['instructions'] = array(
    '#prefix' => '<p>',
    '#markup' => t('The below command can be used in an install profile or update hook to create a field %field_name.', array('%field_name' => $field_name)),
    '#suffix' => '</p>',
  );
  $build['command'] = array(
    '#prefix' => '<textarea cols="100" rows="12" class="form-textarea">',
    '#markup' => '$field = ' . drupal_var_export($field) . ';' . "\n" . 'field_create_field($field);',
    '#suffix' => '</textarea>',
  );
  return $build;
}

/**
 * Export a given field instance.
 */
function fieldhelp_field_export_instance($entity_type, $field_name, $bundle_name) {
  $instance = field_info_instance($entity_type, $field_name, $bundle_name);
  unset($instance['id'], $instance['field_id']);
  $instance = fieldhelp_remove_instance_defaults($instance);
  return $instance;
}

/**
 * Export a given field definition.
 */
function fieldhelp_field_export_field($field_name) {
  $field = field_info_field($field_name);
  unset($field['id']);
  $field = fieldhelp_remove_field_defaults($field);
  return $field;
}

/**
 * Remove field_create_instance() defaults from field_info_instance() array.
 *
 * @param array $instance
 */
function fieldhelp_remove_instance_defaults($instance) {
  $defaults = array(
    'label' => $instance['field_name'],
    'description' => '',
    'deleted' => 0,
    'required' => 0,
    'default_value' => NULL,
    'default_value_function' => '',
  );

  $instance = array_diff_assoc($instance, $defaults);

  $field = field_read_field($instance['field_name']);
  $field_type = field_info_field_types($field['type']);

  // Remove default instance settings.
  $instance['settings'] = array_diff_assoc($instance['settings'], field_info_instance_settings($field['type']));
  if ($instance['settings'] === array()) {
    unset($instance['settings']);
  }

  // Remove default widget and settings.
  if (isset($instance['widget']['type'])) {
    $type = $instance['widget']['type'];
    $instance['widget'] = array_diff_assoc($instance['widget'], array(
      'type' => $field_type['default_widget'],
      'active' => TRUE,
    ));
    $instance['widget']['settings'] = array_diff_assoc($instance['widget']['settings'], field_info_widget_settings($type));
    if ($instance['widget']['settings'] === array()) {
      unset($instance['widget']['settings']);
    }
  }
  // Remove default display settings.
  if (isset($instance['display'])) {
    foreach ($instance['display'] as $view_mode => $actual) {
      $display = array(
        'label' => 'above',
        'type' => isset($field_type['default_formatter']) ? $field_type['default_formatter'] : 'hidden',
      );
      if ($display['type'] != 'hidden') {
        $formatter_type = field_info_formatter_types($display['type']);
        $display['module'] = $formatter_type['module'];
      }
      $instance['display'][$view_mode] = array_diff_assoc($instance['display'][$view_mode], $display);
      if (isset($instance['display']['settings'])) {
        $instance['display']['settings'] = array_diff_assoc($instance['display']['settings'], field_info_formatter_settings($display['type']));
      }
    }
    if ($instance['display'] === array('default' => array())) {
      unset($instance['display']);
    }
  }
  return $instance;
}

function fieldhelp_remove_field_defaults($field) {
  unset($field['bundles']);

  // 'columns' and 'foreign keys' are hardcoded in the field type.
  unset($field['columns'], $field['foreign keys']);

  // Remove default storage information.
  module_load_install($field['module']);
  $schema = (array) module_invoke($field['module'], 'field_schema', $field);
  $schema += array('columns' => array(), 'indexes' => array(), 'foreign keys' => array());
  if (isset($field['indexes'])) {
    foreach ($field['indexes'] as $key => $value) {
      if (isset($schema['indexes'][$key])) {
        $field['indexes'][$key] = array_diff_assoc($value, $schema['indexes'][$key]);
        if ($field['indexes'][$key] === array()) {
          unset($field['indexes'][$key]);
        }
      }
    }
    $field['indexes'] = array_diff_assoc($field['indexes'], $schema['indexes']);
    if ($field['indexes'] === array()) {
      unset($field['indexes']);
    }
  }

  $field_type = field_info_field_types($field['type']);
  $field = array_diff_assoc($field, array(
    'active' => TRUE,
    'cardinality' => TRUE,
    'deleted' => FALSE,
    'locked' => FALSE,
    'module' => $field_type['module'],
    'translatable' => FALSE,
  ));
  // Remove numeric equivalents to default TRUE/FALSE also.
  $field = array_diff_assoc($field, array(
    'active' => 1,
    'cardinality' => 1,
    'deleted' => 0,
    'locked' => 0,
    'translatable' => 0,
  ));

  if (isset($field['entity_types']) && $field['entity_types'] === array()) {
    unset($field['entity_types']);
  }

  if (isset($field['settings'])) {
    $settings = field_info_field_settings($field['type']);
    if (isset($settings['granularity'])) {
      $granularity = $settings['granularity'];
      unset($settings['granularity']);
      $field['settings']['granularity'] = array_diff_assoc($field['settings']['granularity'], $granularity);
    }
    $field['settings'] = array_diff_assoc($field['settings'], $settings);
  }
  if ($field['settings'] === array()) {
    unset($field['settings']);
  }

  if (isset($field['storage']['type'])) {
    if (isset($field['storage']['settings'])) {
      $field['storage']['settings'] = array_diff_assoc($field['storage']['settings'], field_info_storage_settings($field['storage']['type']));
    }
    $storage_type = field_info_storage_types($field['storage']['type']);
    $field['storage'] = array_diff_assoc($field['storage'], array(
      'module' => $storage_type['module'],
      'active' => 1,
    ));
  }
  if (isset($field['storage'])) {
    $field['storage'] = array_diff_assoc($field['storage'], array(
      'type' => variable_get('field_storage_default', 'field_sql_storage'),
    ));
  }
  if ($field['storage']['settings'] === array()) {
    unset($field['storage']['settings']);
  }

  return $field;
}